#include "hardware.h"
#include <em_device.h>

void CMU_setup(void)							// configure clock
{
	CMU->OSCENCMD = CMU_OSCENCMD_HFRCOEN + CMU_OSCENCMD_HFXODIS +
					CMU_OSCENCMD_LFRCOEN + CMU_OSCENCMD_LFXODIS; // enable HFRCO, LFRCO
					
	CMU->LFCLKSEL = CMU_LFCLKSEL_LFA_LFRCO;		// set LFRCO as LFA
	CMU->HFCORECLKEN0 = CMU_HFCORECLKEN0_LE + CMU_HFCORECLKEN0_DMA;	// enable LE clock
	CMU->HFPERCLKEN0 = CMU_HFPERCLKEN0_GPIO + CMU_HFPERCLKEN0_I2C0 + 
					   CMU_HFPERCLKEN0_USART0; // enable HF clock to I2C,GPIO,USART
	CMU->LFACLKEN0 = CMU_LFACLKEN0_RTC + CMU_LFACLKEN0_LETIMER0; // enable LFA to LETIMER and RTC
}

void GPIO_setup(void)							// configure GPIO ports	
{
	(GPIO->P[4]).MODEH = GPIO_P_MODEH_MODE12_WIREDAND +	// PE12/13 = SDA/SCL of I2C0
						 GPIO_P_MODEH_MODE13_WIREDAND;
	(GPIO->P[1]).MODEL = GPIO_P_MODEL_MODE7_PUSHPULL;	// A0  at PB.7
	(GPIO->P[1]).MODEH = GPIO_P_MODEH_MODE8_PUSHPULL;	// RST at PB.8
	(GPIO->P[1]).DOUTSET = 0x900;				// set CS and RST high
}

void LETIMER0_setup(void)						// configure LETIMER
{
	LETIMER0->CTRL = LETIMER_CTRL_DEBUGRUN + LETIMER_CTRL_COMP0TOP; // free mode with TOP=COMP0
	LETIMER0->COMP0 = 0x7FFF;					// 1 sec delay setup
	LETIMER0->IEN = LETIMER_IEN_UF;				// enable underflow interrupt
	LETIMER0->CMD = LETIMER_CMD_START;			// start LETIMER0
	NVIC_ClearPendingIRQ(LETIMER0_IRQn);		// clear pending LPTRM0 interrupt
	NVIC_EnableIRQ(LETIMER0_IRQn);				// enable LPTMR0 interrupt in NVIC
}

void I2C0_setup(void)
{
	I2C0->ROUTE = I2C_ROUTE_LOCATION_LOC6 +		// enable I2C pins at location 6
				  I2C_ROUTE_SDAPEN + I2C_ROUTE_SCLPEN;
	I2C0->CLKDIV = 17;							// set 100 KHz I2C clock (@ PERCLK=14 MHz)	
	NVIC_ClearPendingIRQ(I2C0_IRQn);			// clear pending I2C0 interrupt
	NVIC_EnableIRQ(I2C0_IRQn);					// enable I2C0 interrupt in NVIC	
}

void SPI0_setup(void)
{
	USART0->ROUTE = USART_ROUTE_LOCATION_LOC5 + // enable CLK,TX pins @ location 5
					USART_ROUTE_TXPEN + USART_ROUTE_CLKPEN;
	USART0->CTRL = USART_CTRL_SYNC + USART_CTRL_MSBF 
					+USART_CTRL_TXBIL_HALFFULL; // synch mode, msb first
	USART0->FRAME = USART_FRAME_DATABITS_EIGHT;	// use 8-bit frames
	USART0->CLKDIV = 4 << 6;					// set 3.5 MHz SPI clock (@ PERCLK=14 MHz)	
	USART0->CMD = USART_CMD_TXEN + USART_CMD_MASTEREN + USART_CMD_CLEARTX; // Master mode, enable TX, clear TX

	(GPIO->P[2]).MODEL = GPIO_P_MODEL_MODE0_PUSHPULL;  // configure MOSI pin for USART0
	(GPIO->P[1]).MODEH = GPIO_P_MODEH_MODE11_PUSHPULL+ // CS  at PB.11
						 GPIO_P_MODEH_MODE13_PUSHPULL; // configure SCLK pin for USART0
	NVIC_ClearPendingIRQ(USART0_TX_IRQn);
	NVIC_EnableIRQ(USART0_TX_IRQn);				// enable USART TX interrupt in NVIC		
}

void DMA_setup(void)
{
	(DMA->CH[0]).CTRL = DMAREQ_USART0_TXBL;		// DMA source
	DMA->CONFIG = DMA_CONFIG_EN;				// enable DMA	
	DMA->CHUSEBURSTS = DMA_CHUSEBURSTC_CH0USEBURSTC; // clear CH0 burst request
	DMA->CHREQMASKC = DMA_CHREQMASKS_CH0REQMASKS; 	 // set single byte CH0 transfer
	DMA->CHALTC = DMA_CHALTC_CH0ALTC;			// use primary DMA structure
//	DMA->CHENS = DMA_CHENS_CH0ENS;				// enable CH0
	DMA->IEN = DMA_IEN_CH0DONE;					// enable DMA complete interrupt
	NVIC_ClearPendingIRQ(DMA_IRQn);
	NVIC_EnableIRQ(DMA_IRQn);					// enable DMA interrupt in NVIC
}

