	INCLUDE TG110F32.inc		; CPU register definitions
	PRESERVE8
	THUMB				; use thumb instruction set

A0	EQU	1<<7			; A0 = PB.7
RST	EQU	1<<8			; RST = PB.8
CS	EQU	1<<11			; CS = PB.11
		
	AREA    |.text|, CODE, READONLY	
;-------------------------------------------------------------------------------
I2CMasterWriteBuf	PROC		; R0=address, R1=buf, R2=len	
	EXPORT	I2CMasterWriteBuf
	push	{LR,R4,R11,R12}
	add	R12, R0, R0		; compose write address
	mov	R11, R1			; copy buffer ptr
	mov	R3, R2			; # of bytes to transmit
	movs	R4, #0			; # of bytes to receive
        PUT	0x001, I2Cn_CTRL, I2C0	; clear I2C reset 
	PUT	0xFFCF, I2Cn_IFC	; clear interrupt flags
	PUT	0x021, I2Cn_CMD		; send ABORT,START commands	
        PUT	0x001, I2Cn_IEN		; enable START interrupt  
	wfi				; enter EM1 power mode
        PUT	0, I2Cn_CTRL		; I2C reset			
	pop	{PC,R4,R11,R12}
	ENDP			

I2CMasterReadBuf	PROC		; R0=address, R1=buf, R2=len, R3=reg (0/1)
	EXPORT	I2CMasterReadBuf	; if reg=0 then just reading will be done
	push	{LR,R4,R11,R12}		; ger address to write must be 1st el-t in buf
	add	R12, R0, R0		; compose write address
	mov	R11, R1			; copy buffer ptr
;	movs	R3, #1			; # of bytes to send (slave read register address)
;	movs	R3, #0
	mov	R4, R2			; # of bytes to receive
        PUT	0x001, I2Cn_CTRL, I2C0	; clear I2C reset 
	PUT	0xFFCF, I2Cn_IFC	; clear interrupt flags
	PUT	0x021, I2Cn_CMD		; send ABORT,START commands	
        PUT	0x001, I2Cn_IEN		; enable START interrupt  
	wfi				; enter EM1 power mode
        PUT	0, I2Cn_CTRL		; I2C reset	
	pop	{PC,R4,R11,R12}
	ENDP				

;--------------------------------------------------------------------
SPIWriteBuf	PROC			; R0=buf, R1=length
	EXPORT	SPIWriteBuf
	push	{LR,R4,R11}	
	mov	R11, R0			; copy buffer ptr
	mov	R3, R1			; # of bytes to send
	movs	R4, #0			; # of bytes to receive
	PUT	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUT	CS, GPIO_PB_DOUTCLR, GPIO ; CS down
	PUT	0xF9, USARTn_IFC, USART0  ; clear USART interrupt flags
	PUT	0x02, USARTn_IEN	; enable TXBL interrupt
	wfi
	PUT	CS, GPIO_PB_DOUTSET, GPIO ; CS up
	pop	{PC,R4,R11}
	ENDP

;--------------------------------------------------------------------
I2C0_IRQHandler	PROC
	EXPORT	I2C0_IRQHandler

	ldr	R2, [R0, #I2Cn_IF]	; R1 = I2C0 interrupt flags
STP_isr	tst	R2, #0x100		; STOP condition is issued
	beq	STT_isr			
	ldr	R2, =SCB_SCR		; cancel sleep on exit 
	movs	R1, #0
	str	R1, [R2]
	PUT	0, I2Cn_IEN		; disable I2C interrupts 
	bx	LR

STT_isr	tst	R2, #0x01		; START condition is issued
	beq	RX_isr			
	PUT	0x01, I2Cn_IFC		; clear START IF
	cmp	R3, #0			; transmit bytes?
	bgt	stt1			; YES - proceed
	PUT 	0x20, I2Cn_IEN		; No  - enable RXDATAV interrupt
	add	R12, #1			; compose slave read address		
	strb	R12, [R0, #I2Cn_TXDATA]	; send slave addr+R
	ldr	R2, =SCB_SCR		; set sleep on exit
	movs	R1, #SLEEPONEXIT
	str	R1, [R2]	
	bx	LR	
stt1	
	PUT	0x90, I2Cn_IEN		; enable TXBL,NACK interrupts 
	strb	R12, [R0, #I2Cn_TXDATA]	; send slave addr+W
	ldr	R2, =SCB_SCR		; set sleep on exit
	movs	R1, #SLEEPONEXIT
	str	R1, [R2]
	bx	LR

RX_isr	tst	R2, #0x20		; receive data into I/O
	beq	NAK_isr			; buffer pointed by R11
	ldrb	R1, [R0, #I2Cn_RXDATA]	; R1 = received byte
	strb	R1, [R11], #1		; store it in I2C buffer	
	subs	R4, #1			; decrement RX bytes counter
	beq	rx1			; all bytes received ?
	PUT	0x04, I2Cn_CMD		; NO - send ACK command
	bx	LR
rx1	
	PUT	0x08, I2Cn_CMD		; YES - send NACK command
	b	I2C_end			;  followed by STOP
		
NAK_isr tst	R2, #0x80		; process NACK in TX mode
	beq	TX_isr
	PUT	0x80, I2Cn_IFC		; clear NACK IF
	b	I2C_end	

TX_isr	tst	R2, #0x10		; transmit data from I/O 
	beq	tx2			; buffer pointed by R11
	subs	R3, #1			; decrement TX bytes counter
	blt	tx1	
	ldrb	R1, [R11], #1		; R1 = byte to transmit
	strb	R1, [R0, #I2Cn_TXDATA]	; load it to TX buffer
	bx	LR			
	
tx1	cbz	R4, I2C_end		; receive bytes?	
;	PUT	0x01, I2Cn_IEN		; YES - enable START interrupt
;	PUT	0x01, I2Cn_CMD		;   and send RESTART command
	bx	LR

I2C_end					; end of I2C session
	PUT	0x100, I2Cn_IEN		; enable STOP interrupt
	PUT	0x002, I2Cn_CMD		; send STOP command
tx2	bx	LR
	ENDP
		
;-------------------------------------------------------------------
USART0_TX_IRQHandler	PROC		; SPI_TX handler
	EXPORT	USART0_TX_IRQHandler
	cbz	R3, txh1	
	ldrb	R1, [R11], #1		; R1 = byte to transmit
	str	R1, [R0, #USARTn_TXDATA]; load it to TX buffer
	subs	R3, #1			; decrement TX bytes counter
	itttt	eq			; last byte sent?
	moveq	R1, #0xFF9		
	streq	R1, [R0, #USARTn_IFC] 	; YES - clear USART interrupt flags	
	moveq	R1, #0x01		
	streq	R1, [R0, #USARTn_IEN]	; and enable TXC interrupt
txh0	bx	LR			

txh1	cbz	R4, txh2		; receive bytes?
	ldr	R1, [R0, #USARTn_RXDATA]; clear RX IF
w4rs	ldr	R1, [R0, #USARTn_IF]	; YES - wait for RX flag set
	tst	R1, #0x04
	beq	w4rs
	ldr	R1, [R0, #USARTn_RXDATA]; clear RX IF
	PUT	0xFF9, USARTn_IFC 	; clear USART interrupt flags
	ldr	R3, =NVIC_ICPR		; clear pending IFs in NVIC
	movs	R1, #0x18
	str	R1, [R3]
;	ldr	R11, =Readbuf		; set pointer to receive buffer
	PUT	0x04, USARTn_IEN	; enable RX dava valid interrupt
	str	R1, [R0, #USARTn_TXDATA]; request data	
	bx	LR

txh2	ldr	R3, =SCB_SCR		; cancel sleep on exit 
	movs	R1, #0
	str	R1, [R3]
	PUT	0x00, USARTn_IEN	; disable TX interrupts	
	bx	LR	
	ENDP
	
	ALIGN
	END
		