	INCLUDE TG110F32.inc		; CPU register definitions
	IMPORT SPIWriteBuf
	PRESERVE8
	THUMB				; use thumb instruction set


A0	EQU	1<<7			; A0 = PB.7
RST	EQU	1<<8			; RST = PB.8
CS	EQU	1<<11			; CS = PB.11

	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=8
DMA_c0	SPACE 	16			; DMA channel 0 data structure
	EXPORT	DMA_c0
		
	AREA    |.text|, CODE, READONLY	
LCD_init	PROC
	EXPORT	LCD_init
	push	{LR}
	PUT	RST, GPIO_PB_DOUTCLR, GPIO ; clear RST for 5 msec	
	movw	R0, #35000		; power-up delay
	subs	R0, #1
	bne	.-2	
	PUT	RST, GPIO_PB_DOUTSET, GPIO

	movs	R0, #0x00		; power-up delay
	movt	R0, #0x10
	subs	R0, #1
	bne	.-2

	PUT	USART0+USARTn_TXDATA, 4, DMA_c0 ; destination address
	ldr	R1, =DMA
	str	R0, [R1, #DMA_CTRLBASE] 	
	
	ldr	R0, =LCD_INI		; pointer to init data		
	movs	R1, #16			; data size
;	bl	SPIWriteBuf		; send LCD init data
	bl	LCDWriteDMA
	PUT	A0, GPIO_PB_DOUTSET, GPIO ; set A0 = 1

	bl	Clear_LCD
	pop	{PC}
	ENDP
	
LCDWriteDMA	PROC			; R0=source, R1=length
	EXPORT	LCDWriteDMA	
	ldr	R2, =DMA_c0		; R2 = DMA structure ptr
	sub	R1, #1			; R1 = length-1
	add	R0, R1			; source end address
	str	R0, [R2]		; fill in source ptr in DMA struture	
	ldr	R0, =0xC0000001
	add	R0, R1, LSL #4		; R0 = DMA control
	str	R0, [R2, #8]		; DMA struc is ready
	
	PUT	CS, GPIO_PB_DOUTCLR, GPIO ; CS down
	PUT	0x01, DMA_CHENS, DMA 	; start DMA transfer
	wfi
	ldr	R0, =USART0
lwd	ldr	R1, [R0, #USARTn_IF]	; wait for IFC set
	lsrs	R1, #1
	bcc	lwd
	PUT	CS, GPIO_PB_DOUTSET, GPIO ; CS up
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
Clear_LCD	PROC			; use DMA to send 0s as LCD data
	EXPORT	Clear_LCD
	push	{LR,R5}	
	ldr	R2, =DMA_c0		; R2 = DMA structure ptr
	ldr	R1, =ZERO		; source end address
	str	R1, [R2]		; fill in source ptr in DMA struture		

	movs	R5, #3			; page #	
cl1	mov	R2, R5			; R2: 0 0 0 y
	bl	Set_Cursor		; set page & column

	ldr	R2, =DMA_c0		; R2 = DMA structure ptr
	ldr	R0, =0xCC0007F1		; R0 = DMA control
	str	R0, [R2, #8]		; DMA struc is ready
	
	PUT	CS, GPIO_PB_DOUTCLR, GPIO ; CS down
	PUT	0x01, DMA_CHENS, DMA 	; start DMA transfer
	wfi
	ldr	R0, =USART0
cl2	ldr	R1, [R0, #USARTn_IF]	; wait for IFC set
	lsrs	R1, #1
	bcc	cl2
	PUT	CS, GPIO_PB_DOUTSET, GPIO ; CS up
	
	subs	R5, #1
	bpl	cl1
	pop	{PC,R5}	
	ENDP

;---------------------------------------------------------------------	
Set_Cursor	PROC			; R2: 0 0 x y
 	push	{LR}
	sub	SP, #4			; sererve place for 3 command bytes
	add	R1, R2, #0xB0		; add page address set cmd
	strb	R1, [SP, #2]	
	and	R1, R2, #0x0F00		; ls nibble(x)
	add	R1, R2, LSR #12
	add	R1, #0x10		; add column set address cmd
	strh	R1, [SP]

	PUT	A0, GPIO_PB_DOUTCLR, GPIO ; set A0 = 0
	mov	R0, SP
	movs	R1, #3			; send 3 bytes
;	movs	R4, #0			; do not receive bytes
	bl	SPIWriteBuf	
	PUT	A0, GPIO_PB_DOUTSET, GPIO ; set A0 = 1	
	add	SP, #4
	pop	{PC}
	ENDP


;---------------------------------------------------------------------	
Display_Digit	PROC			; R0=digit, R1=x
	EXPORT	Display_Digit		 
	push	{LR,R5-R9}	
	ldr	R5, =Font		; font array ptr
	movs	R2, #19*4		; font digit size
	mla	R5, R0, R2, R5		; R5 = offset in font array
	mov	R6, R1			; R6 = x
	movs	R7, #19			; font width
	movs	R8, #4			; font height 
	movs	R9, #3			; starting page
	b	dbm1
	
Display_Bitmap				; R0=address, R1=x, R2=page 
	push	{LR,R5-R9}
	mov	R6, R1			; R6 = x	
	ldrb	R7, [R0], #1		; R7 = width	
	ldrb	R8, [R0], #1		; R8 = height (in pages)
	mov	R9, R2			; R2 = y
	mov	R5, R0			; R5 = init bitmap address
dbm1	add	R2, R9, R6, LSL #8	; R2 = cursor position
	bl	Set_Cursor		; set page & column
		
	mov	R0, R5			; source address
	mov	R1, R7			; # of bytes to send
	bl	LCDWriteDMA
	add	R5, R7			; advance bitmap pointer

	subs	R9, #1			; get to lower page
	subs	R8, #1			; loop over bitmap pages
	bgt	dbm1
	pop	{PC,R5-R9}
	ENDP

	ALIGN	
	LTORG
;----------------------------------------------------------------------------
Display_Temp_Humi	PROC
	IMPORT	temp
	IMPORT	humi	
	EXPORT	Display_Temp_Humi
	push	{LR}
	bl	Clear_LCD
	ldr	R3, =humi
	ldrb	R1, [R3]		; get humi value
	movs	R2, #10
	udiv	R0, R1, R2		; R0 = R1/10
	mls	R1, R0, R2, R1		; R1 = R1%10
	push	{R1}			; save it in stack		
	movs	R1, #0			; x-coord.
	bl	Display_Digit		; display R0=tens of humi

	pop	{R0}			; R0 = units of humi
	movs	R1, #21			; x-coord
	bl	Display_Digit		; display R0=units of humi
	
	ldr	R0, =Percent		; '%' symbol pointer
	movs	R1, #43			; column #
	movs	R2, #3			; page #
	bl	Display_Bitmap		
;-----------------------------------------------	
	ldr	R3, =temp
	ldrb	R1, [R3]		; get temp value
	sxtb	R1, R1			; sign extend it to word
	ldr	R0, =Sign
	cmp	R1, #0
	bge	above
	
	rsbs	R1, R1, #0		; negate temp if negative
	adds	R0, #8			; pointer to '-'
above
	push	{R1}			; save R1
	movs	R1, #61			; column #
	movs	R2, #1			; page #
	bl	Display_Bitmap		
	pop	{R1}			; restore R1=temp

	movs	R2, #10
	udiv	R0, R1, R2		; R0 = R1/10
	mls	R1, R0, R2, R1		; R1 = R1%10
	push	{R1}			; save it in stack	
	movs	R1, #69			; x-coord
	movs	R2, #3			; y-coord
	bl	Display_Digit		; display R0=tens of temp
	
	pop	{R0}			; R0 = units of temp
	movs	R1, #90			; x-coord
	movs	R2, #3			; y-coord
	bl	Display_Digit		; display R0=units of temp

	ldr	R0, =Deg_C		; 'C' symbol pointer
	movs	R1, #111		; column #
	movs	R2, #3			; page #			
	bl	Display_Bitmap
	pop	{PC}
	ENDP
		
;------------------------------------------------------------------------
	ALIGN
LCD_INI	
	DCW	0x00AC, 0xA5AF, 0xA0A2, 0x20C0, 0x3781, 0xA42F, 0x01AC, 0xAF40
ZERO
	DCB	0x00, 0x00, 0x00, 0x00
;----------------------------------------------------------------------
Font	; 19x32 font
	EXPORT	Font
	DCB	0x00, 0x03, 0x0F, 0x1F, 0x3F, 0x7F, 0x7F, 0xFF, 0xE0, 0xC0, 0xE0	; 0
	DCB	0xFF, 0x7F, 0x7F, 0x3F, 0x1F, 0x0F, 0x03, 0x00
	DCB	0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x3F
	DCB	0xFC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC
	DCB	0x00, 0xC0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFE, 0xFF, 0x07, 0x03, 0x07
	DCB	0xFF, 0xFE, 0xFE, 0xFC, 0xF8, 0xF0, 0xC0, 0x00

	DCB	0x00, 0x00, 0x01, 0x01, 0x03, 0x03, 0x07, 0x0F, 0x1F, 0x7F, 0xFF	; 1
	DCB	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0x00, 0x00, 0xC0, 0xC0, 0xC0, 0xC0, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0x07, 0x07, 0x00, 0x00, 0x00

	DCB	0x07, 0x1F, 0x3F, 0x7F, 0x7F, 0xFF, 0xFF, 0xE0, 0xC0, 0xC0, 0xE0	; 2
	DCB	0xFF, 0xFF, 0x7F, 0x7F, 0x3F, 0x1F, 0x0F, 0x03
	DCB	0x80, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xC0, 0x00, 0x00, 0x01, 0x0F
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFC, 0xF0
	DCB	0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x1F, 0x3E, 0x7E, 0xFC, 0xF8
	DCB	0xF0, 0xE0, 0xE0, 0xC0, 0x81, 0x01, 0x01, 0x01
	DCB	0x3F, 0x7F, 0xFF, 0xFF, 0xFF, 0xBF, 0x3F, 0x3F, 0x3F, 0x3F, 0x3F
	DCB	0x3F, 0x3F, 0x3F, 0x7F, 0xFF, 0xFF, 0xFE, 0xE0

	DCB	0x0F, 0x1F, 0x3F, 0x7F, 0x7F, 0xFF, 0xE0, 0xC0, 0xC0, 0xE0, 0xF0	; 3
	DCB	0xFF, 0x7F, 0x7F, 0x7F, 0x3F, 0x1F, 0x07, 0x00
	DCB	0x80, 0xC0, 0xC0, 0xC0, 0xC3, 0x83, 0x03, 0x03, 0x03, 0x07, 0x0F
	DCB	0xFF, 0xFF, 0xFD, 0xFC, 0xF8, 0xF0, 0xC0, 0x00
	DCB	0x01, 0x03, 0x03, 0x03, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0xE0
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x3F, 0x0F
	DCB	0xF0, 0xF8, 0xFC, 0xFE, 0xFE, 0xFF, 0x07, 0x03, 0x03, 0x07, 0x0F
	DCB	0xFF, 0xFE, 0xFE, 0xFE, 0xFC, 0xF8, 0xF0, 0xC0

	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03	; 4
	DCB	0x0F, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00
	DCB	0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x3E, 0x78, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00
	DCB	0x07, 0x0F, 0x3F, 0x7F, 0xFB, 0xE3, 0x83, 0x03, 0x03, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x03, 0x03
	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x07, 0x07

	DCB	0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC	; 5
	DCB	0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0x00, 0x00
	DCB	0x00, 0x3F, 0xFF, 0xFF, 0xFF, 0x1F, 0x1C, 0x1C, 0x1C, 0x1E, 0x1F
	DCB	0x1F, 0x0F, 0x0F, 0x0F, 0x07, 0x03, 0x01, 0x00
	DCB	0x01, 0x83, 0x83, 0x83, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x3F
	DCB	0xF0, 0xF8, 0xFC, 0xFE, 0xFE, 0xFF, 0x03, 0x03, 0x03, 0x03, 0x07
	DCB	0xFF, 0xFE, 0xFE, 0xFC, 0xF8, 0xF0, 0xE0, 0x80

	DCB	0x00, 0x00, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x3F, 0x7C, 0x70, 0xF0	; 6
	DCB	0xE0, 0xE0, 0xC0, 0xC0, 0xC0, 0xC0, 0x00, 0x00
	DCB	0x1F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x70, 0x70, 0x70
	DCB	0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x1F, 0x07, 0x01
	DCB	0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	DCB	0x00, 0xE0, 0xF0, 0xFC, 0xFE, 0xFE, 0xFF, 0xFF, 0x07, 0x03, 0x07
	DCB	0xFF, 0xFE, 0xFE, 0xFC, 0xFC, 0xF8, 0xE0, 0x00

	DCB	0x3F, 0xFF, 0xFF, 0xFF, 0xFE, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC	; 7
	DCB	0xFC, 0xFC, 0xFF, 0xFF, 0xFF, 0xFE, 0xF8, 0xE0
	DCB	0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x0F
	DCB	0x3F, 0xFF, 0xFF, 0xF8, 0xC0, 0x00, 0x00, 0x00
	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x1F, 0xFF, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	DCB	0xFF, 0xFF, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00

	DCB	0x00, 0x07, 0x1F, 0x3F, 0x7F, 0x7F, 0xFF, 0xF0, 0xE0, 0xC0, 0xC0	; 8
	DCB	0xE0, 0xF0, 0x7F, 0x7F, 0x3F, 0x1F, 0x07, 0x00
	DCB	0x00, 0xE0, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x3F, 0x1F
	DCB	0x0F, 0x3F, 0xFF, 0xFF, 0xF9, 0xF0, 0xC0, 0x00
	DCB	0x07, 0x1F, 0x3F, 0x7F, 0xFF, 0xFF, 0xF0, 0xE0, 0xF0, 0xF8, 0xF8
	DCB	0xFC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x1F
	DCB	0xC0, 0xF0, 0xFC, 0xFC, 0xFE, 0xFE, 0x1F, 0x07, 0x03, 0x03, 0x03
	DCB	0x07, 0x0E, 0xFE, 0xFE, 0xFC, 0xF8, 0xF0, 0xC0

	DCB	0x00, 0x07, 0x0F, 0x3F, 0x7F, 0x7F, 0xFF, 0xFF, 0xC0, 0xC0, 0xC0	; 9
	DCB	0xFF, 0xFF, 0x7F, 0x7F, 0x3F, 0x1F, 0x07, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F
	DCB	0x80, 0xE0, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0x1E, 0x0E, 0x1C
	DCB	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF8
	DCB	0x00, 0x00, 0x03, 0x03, 0x03, 0x03, 0x07, 0x07, 0x0E, 0x0E, 0x3E
	DCB	0xFC, 0xFC, 0xF8, 0xF8, 0xF0, 0xC0, 0x00, 0x00	

Percent
	DCB	0x0D, 0x02 	; w x h of '%'
	DCB	0x78, 0xFC, 0x84, 0x84, 0xFC, 0x78, 0x01, 0x03, 0x06, 0x0C, 0x18, 0x30, 0x60
	DCB	0x04, 0x08, 0x18, 0x30, 0x60, 0xC0, 0x80, 0x1E, 0x3F, 0x21, 0x21, 0x3F, 0x1E

Deg_C
	DCB	0x11, 0x02 	; w x h of 'C'
	DCB	0x78, 0xFC, 0x84, 0x84, 0xFC, 0x78, 0x00, 0x0F, 0x3F, 0x7F, 0xE0
	DCB	0xC0, 0x80, 0x80, 0x80, 0xC0, 0xC0
	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0xF8, 0xFC, 0x0E
	DCB	0x06, 0x02, 0x02, 0x02, 0x06, 0x04

Sign
	DCB	0x06, 0x01	; erase minus
	DCB	0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	DCB	0x06, 0x01	; w x h of '-'
	DCB	0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0
	
	END